/****************************************************************************
 *
 * @file	main.c
 * @brief	This is a boot loader sample program for S1C17W34/W35/W36 demonstration.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2015-2018. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 02/01/2018    1.00
 *				- First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "srec.h"
#include "fl_self.h"

#include "crt0.h"
#include "init.h"
#include "clg.h"
#include "uart2.h"
#include "t16_ch1.h"

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#define C17_INTERRUPT_(vh, ih)		void vh(void) __attribute__ ((unused,alias(#ih)))
C17_INTERRUPT_(_vector28_handler, start_upgrade);
#endif

/* --- definition --- */
#define		BUFF_SIZE		(256)
#define		BUFF_SIZE_HALF	(BUFF_SIZE>>1)
#define		XON_CODE		(0x11)
#define		XOFF_CODE		(0x13)

#if defined(C17_MCUSEL_W36)
	#define		ROM_END			0x67FFF			//W36
	#define		RAM_END			0x3FC0			//W36
	#define		SECTOR_MAX		96				//W36
#endif

#if defined(C17_MCUSEL_W35)
	#define		ROM_END			0x47FFF			//W35
	#define		RAM_END			0x2FC0			//W35
	#define		SECTOR_MAX		64				//W35
#endif

#if defined(C17_MCUSEL_W34)
	#define		ROM_END			0x27FFF			//W34
	#define		RAM_END			0x2FC0			//W34
	#define		SECTOR_MAX		32				//W34
#endif

/* --- extern --- */
extern char __START_program[];
extern const unsigned char upgrade_indicator[4];
extern int (*flentry[])();


/* --- struct --- */
typedef struct st_data_buf {
	unsigned short end_point;				/// buff end point
    char buf[BUFF_SIZE];
} ST_DATA_S;

/* --- static variable --- */
static unsigned char do_upgrade[4] = {
	0x00, 0x00, 0x00, 0x00,
};
static unsigned char upgrade_completed[4] = {
	0xaa, 0xaa, 0xaa, 0xaa,
};

static struct srec_decode_t srecData;		/// decode data buffer
static ST_DATA_S 	oneLineData;			/// UART receive buffer

/* --- function declaration --- */
int main(void);
static int memory_write(struct srec_decode_t * srec);
static void data_copy(void);
static int get_record(void);
static int update(void);
static void start_wdt(void);
static void stop_wdt(void);


int main(void)
{
	int result;

	/// STOP WDT
	stop_wdt();

	/// Iram size select
#if defined(C17_MCUSEL_W35) || 	defined(C17_MCUSEL_W34)
	setMiscIRAMSize(MSCIRAMSZ_12KB);
#endif
#if defined(C17_MCUSEL_W35)
	setMiscIRAMSize(MSCIRAMSZ_16KB);
#endif

	if (memcmp(upgrade_indicator, upgrade_completed, 4) == 0) {
		/// Set Vector table base address = 0xC000
		setMiscVectorAddress( (unsigned long) __START_program);
		/// jump to the reset vector handler of the loaded program.
		asm("xld.a %r0, [__START_program]");
		asm("jpa %r0");
	}
	if (memcmp(upgrade_indicator, do_upgrade, 4) == 0) {
		/// erase sector (0xC000 -ROM_END )
		(*flentry[0])((char *)0x8000, 5, SECTOR_MAX);
		/// erase sector 4 (0xB000)
		(*flentry[0])((char *)0x8000, 4, 4);
	}

	/// receive S-record and write it.
	result = update();

	while(1)
	{};

	return result;
}

/****************************************************************************
 * start upgrade function.
 *
 * @brief	start upgrade.
 *
 * @retval	none
 ***************************************************************************/
void start_upgrade(void)
{
	unsigned short ie;
	unsigned char data[] = {0x00, 0x00, 0x00, 0x00};
	/// Get IE Flag
	ie = MSCPSR_PSRIE;

	/// interrupt disable
	asm("di");

	/// Flash Write
	(*flentry[1])((char *)0xb000, 4, (unsigned char*)data);


	/// restore IE flag
	if(ie == 1){
		asm("ei");
	}
	else{
		asm("di");
	}
}

/****************************************************************************
 * update function.
 *
 * @brief	update function.
 * UART setting
 *	port
 *	 P00 : SOUT
 *	 P01 : SIN
 *
 *	baud rate	: 230400bps
 *	data bits	: 8bit
 *	parity		: none
 *	stop bit	: 1bit
 *	flow control: xon/xoff
 *
 *
 * @retval	EXIT_SUCCESS:Success.
 * @retval	EXIT_FAILURE:Error.
 ***************************************************************************/
int update(void)
{
	short srecResult;

	/// Change clock source=OSC3(4MHz)
	startClgOSC3f( CLG_OSC3_SEL_4MHZ );
	switchClgSystemClockOSC3();
	stopClgIOSC();

	/// initialize for receive buffer.
	srecData.record_length = 0;
	oneLineData.end_point = 0;

	/// initialize T16 Ch.1
	initT16Ch1();
	/// initialize UART Ch.0
	initUart();
	/// start UART
	startUart();

	while(1)
	{
		/// Get S record from PC terminal.
		if(get_record() == EXIT_FAILURE)	{
			continue;
		}
		/// S Record => Decode
		srecResult = srec_decode_record(&srecData, oneLineData.buf);
		switch(srecResult)
		{
			case SREC_TYPE_S0:
			case SREC_TYPE_S4:
			case SREC_TYPE_S5:
			case SREC_TYPE_S6:
				break;
			case SREC_TYPE_S1:
			case SREC_TYPE_S2:
			case SREC_TYPE_S3:
				/// Flash Write(data)
				if(memory_write(&srecData) != FLS_E_SUCCESS)	{
					/// Error
					stopUart();
					return EXIT_FAILURE;
				}
				break;
			case SREC_TYPE_S7:
			case SREC_TYPE_S8:
			case SREC_TYPE_S9:
				/// Flash Write(upgrade_completed)
				(*flentry[1])((char *)0xb000, 4, (unsigned char*)upgrade_completed);
				goto endRecv;
				break;
			case SREC_TYPE_UNKNOWN:
			case SREC_TYPE_ERROR_CHECKSUM:
			case SREC_TYPE_ERROR_LENGTH:
				/// Decode Error
				stopUart();
				return EXIT_FAILURE;
				break;
		}
		/// remaining data copy
		data_copy();
	}

endRecv:
	/// stop UART Ch.0
	stopUart();

	/// IC reset
	start_wdt();

	return EXIT_SUCCESS;
}
/****************************************************************************
 * data_copy function.
 *
 * @brief	Remaining data copy.
 *
 * @retval	none
 ***************************************************************************/
void data_copy(void)
{
	int remainSize;

	if(srecData.record_length == 0){
		return;
	}
	remainSize = oneLineData.end_point - srecData.record_length -1;
	memcpy(oneLineData.buf, oneLineData.buf + srecData.record_length + 1, remainSize);
	srecData.record_length = 0;
	oneLineData.end_point = remainSize;

}

/****************************************************************************
 * Get S Record Receive function.
 *
 * @brief	Get S record data from PC(UART).
 *
 * @retval	EXIT_SUCCESS:Success.
 ***************************************************************************/
int get_record(void)
{
	int i;
	char * buffer;
	char data;
	unsigned short cr_flag;
	unsigned short end_point;

	buffer = oneLineData.buf + oneLineData.end_point;
	cr_flag = 0;
	end_point = oneLineData.end_point;

	/// preset T16 Ch.1
	T16_1TR = 0x7c;							/// - OSC3=4MHz, CLKDIV=1/16 [(124+1)/(4MHz/16)=0.5msec.]

	/// start data receive
	UA0TXD = XON_CODE;

	/// 3msec * 10000 = 30sec
	for (i = 0; i < 10000; i++) {
		///timer start
		T16_1INTF = 0x0001;
		T16_1CTL = 0x0103;
		do {
			if (UA0INTF_RB1FIF != 0) {
				*buffer++ = UA0RXD;
				end_point++;
				goto START;
			}
		} while (T16_1INTF_UFIF == 0);
	}
	return EXIT_FAILURE;
START:
	do {
		/// Check receive buffer one byte full interrupt.
		if (UA0INTF_RB1FIF != 0) {
			/// save receive data.
			data = UA0RXD;
			end_point++;
			if (data == 0x0d) {
				*buffer++ = 0;
				/// stop data receive
				UA0TXD = XOFF_CODE;
				cr_flag = 1;
				break;
			} else {
				*buffer++ = data;
			}
			T16_1INTF = 0x0001;
			T16_1CTL = 0x0103;
		}
	} while (T16_1INTF_UFIF == 0);


	/// Receive remaining data.
	do {
		/// Check receive buffer one byte full interrupt.
		if (UA0INTF_RB1FIF != 0) {
			/// save receive data.
			data = UA0RXD;
			end_point++;
			if (data == 0x0d) {
				*buffer++ = 0;
				break;
			} else {
				*buffer++ = data;
			}
			T16_1INTF = 0x0001;
			T16_1CTL = 0x0103;
		}
	} while (T16_1INTF_UFIF == 0);


	oneLineData.end_point = end_point;
	if(cr_flag == 0){
		return EXIT_FAILURE;
	}
	else{
		return EXIT_SUCCESS;
	}
}

/****************************************************************************
 * memory_write function.
 *
 * @brief	Flash Write.
 *
 * @retval	FLS_E_SUCCESS: Success.
 ***************************************************************************/
int memory_write(struct srec_decode_t * srec)
{
	int result;

	if((srec->address >= 0xC000)&& (srec->address < ROM_END)){
		/// Flash Write
		result = (*flentry[1])((char *)srec->address, srec->length, srec->data);
	}
	else if((srec->address >= 0xC00) && (srec->address < RAM_END)){
		/// Sram Write
		memcpy((char *)srec->address, srec->data, srec->length);
		result = FLS_E_SUCCESS;
	} else {
		result = FLS_E_SUCCESS;
	}

	return result;
}
/****************************************************************************
 * start_wdt function.
 *
 * @brief	Reset by WDT.
 *
 * @retval	none
 ***************************************************************************/
void start_wdt(void)
{
	protectMisc( MSCPROT_PROTECT_OFF );
	// WDT2 Clock Control Register
	WDTCLK = 0x0002;			/// Clock source = OSC3/8192

	// WDT2 Counter Compare Match Register.
	WDTCMP = 0x3FF;				/// Reset wait 2sec[4Mhz/8192*2sec]

	// WDT2 Control Register
	WDTCTL = 0x0010;			/// Reset mode,start
	protectMisc( MSCPROT_PROTECT_ON );

}
/****************************************************************************
 * stop_wdt function.
 *
 * @brief	Reset by WDT.
 *
 * @retval	none
 ***************************************************************************/
void stop_wdt(void)
{
	protectMisc( MSCPROT_PROTECT_OFF );
	// WDT2 Control Register
	WDTCTL = 0x001A;			/// stop
	protectMisc( MSCPROT_PROTECT_ON );
}
