/****************************************************************************
 *
 * @file	t16_ch1.c
 * @brief	This is 16-bit Ch.1 Timers driver header file
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 05/19/2016    1.00
 *				- First revision.
 *
 ***************************************************************************/

#include "t16_ch1.h"

/****************************************************************************
 * initT16Ch1 function.
 *
 * @brief	T16 Ch.1 timer initialize.
 *
 * @retval	TRUE	Success.
 * @retval	FALSE	Failed.
 ***************************************************************************/
int initT16Ch1( void )
{
	/// Check IOSC.
	if ( CLGOSC_OSC3EN == 0 ) {
		return FALSE;
	}
	/// Stop T16 Ch.1 Timer.
 	stopT16Ch1();

	T16_1CTL			&= 0xfffe;			/// - Stop supplying operating clock

	/// Configure the T16 Ch.1 operating clock.
	T16_1CLK		 	= 0x0042;
	//T16_1CLK		 	= 0x0042;
	//T16_1CLK_CLKSRC 	= 2;				/// - Clock source selection(setting OSC3:4MHz)
	//T16_1CLK_CLKDIV 	= 4;				/// - Clock division ratio selection(setting 1/16)
	//T16_1CLK_DBRUN	= 0;				/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_1CTL			= 0x0001;			/// - Start supplying operating clock.

	/// Select operation mode.
	T16_1MOD			= 1;				/// - Set one shot mode.

	/// initialize interrupt.
	T16_1INTF			= 0x0001;			/// - Clear interrupt flag(T16_0INTF_UFIF = 1).
	T16_1INTE			= 0;				/// - Disable interrupt.
	ILVT16_1			= 0;				/// - Set interrupt level-0.

	/// Preset reload data to counter.
	presetT16Ch1Count(0x1d4b);				/// - OSC3=4MHz, CLKDIV=1/16 [(7499+1)/(4MHz/16)=30msec.]
	//presetT16Ch1Count(0x9c3);				/// - OSC3=4MHz, CLKDIV=1/16 [(24999+1)/(4MHz/16)=10msec.]
	//presetT16Ch1Count(0x30d3);			/// - OSC3=4MHz, CLKDIV=1/16 [(12499+1)/(4MHz/16)=50msec.]
	//presetT16Ch1Count(0x4e1f);			/// - OSC3=4MHz, CLKDIV=1/16 [(19999+1)/(4MHz/16)=80msec.]

	return TRUE;
}

/****************************************************************************
 * startT16Ch0  function.
 *
 * @brief	T16 Ch.1 timer start.
 ***************************************************************************/
void startT16Ch1(void)
{
	/// Clear interrupt flag.
	//T16_0INTF_UFIF		= 1;
	T16_1INTF			= 0x0001;
	/// Enable T16 Ch.1 underflow interrupts.
	//T16_1INTE			= 1;
	/// T16_0CTL.PRUN set 1(timer start).
	T16_1CTL			|= 0x0100;
}

/****************************************************************************
 * stopT16Ch1 function.
 *
 * @brief	T16 Ch.1 timer stop.
 ***************************************************************************/
void stopT16Ch1(void)
{
	/// T16_1CTL.PRUN set 0(timer stop).
	T16_1CTL			&= 0xfeff;
	/// Disable T16 Ch.1 underflow interrupts.
	T16_1INTE			= 0;
	/// Clear interrupt flag.
	//T16_0INTF_UFIF		= 1;
	T16_1INTF			= 0x0001;
}

/****************************************************************************
 * presetT16Ch1Count function.
 *
 * @brief	 Set T16 Ch.1 timer count value.
 *
 * @param	count		Counter Data.
 ***************************************************************************/
void presetT16Ch1Count( unsigned short count )
{
	/// Set the initial value to be preset to the counter.
	T16_1TR = count;

	/// Presets the reload data stored in the T16_0TR register to the counter.
	T16_1CTL |= 0x0002;
}
