/****************************************************************************
 *
 * @file	uart2.c
 * @brief	This is UART2 driver program file for S1C17W35 demonstration.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 10/11/2016    1.00
 *				- First revision.
 *
 ***************************************************************************/

#include "uart2.h"
#include "t16_ch1.h"

static unsigned short getRxData( void );

/****************************************************************************
 * initUart function.
 *
 * @brief	initialize UART.
 *
 * @retval	TRUE	Success.
 * @retval	FALSE	Failed.
 ***************************************************************************/
int initUart( void )
{
	/// When OSC3 doesn't start, nothing is done.
	if ( CLGOSC_OSC3EN == 0 ) {
		return FALSE;
	}

	/// Stop UART.
	stopUart();

	/// Initialize the UART input/output function to the ports.
	P0IOEN				&= 0xfcfc;
	P0MODSEL			&= 0xfffc;
	//P0IOEN_P00IEN		= 0;			/// - P00 : Disable input.
	//P0IOEN_P001OEN	= 0;			/// - P00 : Disable output.
	//P0MODSEL_P00SEL	= 0;			/// - P00 : Disable peripheral I/O function.
	//P0IOEN_P01IEN		= 0;			/// - P01 : Disable input.
	//P0IOEN_P01OEN		= 0;			/// - P01 : Disable output.
	//P0MODSEL_P01SEL	= 0;			/// - P01 : Disable peripheral I/O function.

	/// Set Universal Port Multiplexer (UPMUX).
	P0UPMUX0			|= 0x2343;
	//P0UPMUX0_P00PERISEL	= 3;		/// - P00 : UART.
	//P0UPMUX0_P00PPFNC		= 2;		/// - P00 : USOUT.
	//P0UPMUX0_P00PERICH	= 0;		/// - P00 : Ch.0.
	//P0UPMUX0_P01PERISEL	= 3;		/// - P01 : UART.
	//P0UPMUX0_P01PPFNC		= 1;		/// - P01 : USIN.
	//P0UPMUX0_P01PERICH	= 0;		/// - P01 : Ch.0.

	/// Assign the UART input/output function to the ports.
	P0FNCSEL			|= 0x0005;
	P0MODSEL			|= 0x0003;
	//P0FNCSEL_P00MUX	= 1;			/// - P00 : UPMUX.
	//P0MODSEL_P00SEL	= 1;			/// - P00 : Enable peripheral I/O function.
	//P0FNCSEL_P01MUX	= 1;			/// - P01 : UPMUX.
	//P0MODSEL_P01SEL	= 1;			/// - P01 : Enable peripheral I/O function.

	/// Configure operating clock.
	UA0CLK				= 0x0102;
	//UA0CLK_CLKSRC		= 2;			/// - Select the clock source of the UART(OSC3=4MHz).
	//UA0CLK_CLKDIV		= 0;			/// - Select the division ratio(1/1).
	//UA0CLK_DBRUN		= 1;			/// - Clock supplied in DEBUG mode.

	/// The clock source operations in SLEEP mode.
	CLGOSC				&= 0xfeff;
	//CLGOSC_IOSCSLPC	= 0;			/// - Continue operation state before SLEEP.

	/// Configure the following UA0MOD register bits.
	UA0MOD				= 0x0448;
	//UA0MOD_BRDIV		= 1;			/// - UART3 operating clock division ratio( 0=1/16 / 1=1/4)
	//UA0MOD_INVRX		= 0;			/// - Set USIN0 inverting(0=Disable input inverting / 1=Enable input inverting)
	//UA0MOD_INVTX		= 0;			/// - Set USOUT0 inverting(0=Disable output inverting / 1=Enable output inverting)
	//UA0MOD_PUEN		= 1;			/// - Set USIN0 pin(0=Disable pull-up / 1=Enable pull-up).
	//UA0MOD_OUTMD		= 0;			/// - Set USOUT0 pin(0=Push-pull output / 1=open-drain output).
	//UA0MOD_IRMD		= 0;			/// - IrDA interface(0=Disable / 1=Enable).
	//UA0MOD_CHLN		= 1;			/// - Data length(0=7-bit  / 1=8-bit).
	//UA0MOD_PREN		= 0;			/// - Parity function(0=Disable / 1=Enable).
	//UA0MOD_PRMD		= 0;			/// - Even parity selection(0=Unused / 1=Used).
	//UA0MOD_STPB		= 0;			/// - Stop bit length(0=1-bit  / 1=2-bit).

	/// Set transfer rate( bps = CLK_UART3 / {(BRT + 1)/BRDIV + FMD} ).
	/// 19200 bps
	//UA0BR				= 0x0432;
	//UA0BR_FMD			= 0x4;			///	- FMD : 4
	//UA0BR_BRT			= 0x32;			/// - BRT : 50

	/// Set transfer rate( bps = CLK_UART3 / {(BRT + 1)/BRDIV + FMD} ).
	/// 57600 bps
	//UA0BR				= 0x020f;
	//UA0BR_FMD			= 0x2;			///	- FMD : 2
	//UA0BR_BRT			= 0x0f;			/// - BRT : 15

	/// Set transfer rate( bps = CLK_UART3 / {(BRT + 1)/BRDIV + FMD} ).
	/// 115200 bps
	//UA0BR				= 0x0307;
	//UA0BR_FMD			= 0x3;			///	- FMD : 3
	//UA0BR_BRT			= 0x07;			/// - BRT : 7

	/// Set transfer rate( bps = CLK_UART3 / {(BRT + 1)/BRDIV + FMD} ).
	/// 230400 bps
	UA0BR				= 0x0103;
	//UA0BR_FMD			= 0x1;			///	- FMD : 1
	//UA0BR_BRT			= 0x03;			/// - BRT : 3

	/// Set interrupt level.
	ILVUART2_0			= 0;			/// - Set interrupt level-0.

	return TRUE;
}

/****************************************************************************
 * startUart function.
 *
 * @brief	Set UART enable.
 ***************************************************************************/
void startUart( void )
{
	/// clear flag
	getRxData();			// RXD dummy read

	UA0INTF |= 0x0078;		// clear flag

	/// Execute software reset.
	UA0CTL		|= 0x0002;

	/// Enable UART Ch.0 operations.
	UA0CTL		|= 0x0001;
}

/****************************************************************************
 * stopUart function.
 *
 * @brief	Set UART enable.
 ***************************************************************************/
void stopUart( void )
{
	/// Disable UART Ch.0 operations.
	UA0CTL		&= 0xfffe;

	/// Execute software reset.
	UA0CTL		|= 0x0002;

}

/****************************************************************************
 * getRxData function.
 *
 * @brief	 dummy read Rx data.
 *
 * @return 	data	Return Rx data.
 ***************************************************************************/
static unsigned short getRxData( void )
{
	return ( UA0RXD );
}
