/*******************************************************************************
 *
 * @file	boot.c
 * @brief	This is vector and interrupt program with C.
 *
 * @par		Copyright:
 * 			Copyright (C) SEIKO EPSON CORPORATION 2015-2015. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 26 Feb, 2015	1.00
 *				- First revision.
 *			- 10 Dec, 2015	1.01
 *				- Change source code.
 *
 ******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <libstdio.h>



/* --- definition --- */

/* -----------------------------------------------------------------------------
 * 	When simulated I/O is used, the following are made effective.
 * 	Please confirm "GNU17 GDB commands" and "GNU17 linker script setting"
 * 	that exists in the property of the project.
 * -------------------------------------------------------------------------- */
//#define SIM_IO_USED


/* --- function declaration --- */

void boot( void );


static void clearBss(void);
static void loadLmaToVmaData(void);
#ifdef SIM_IO_USED
static void loadLmaToVmaSimIO(void);
#endif // SIM_IO_USED
static void end(void);

// Non Maskable Interrupt Handler
void intAddrErr(void)			__attribute__ ((interrupt_handler));
void intReserved(void)		__attribute__ ((interrupt_handler));

/* --- external declaration --- */

extern int main(void);
extern void start_upgrade(void);

extern unsigned long __START_bss;
extern unsigned long __END_bss;
extern unsigned long __START_data;
extern unsigned long __START_data_lma;
extern unsigned long __END_data;
extern unsigned long __START_simio;
extern unsigned long __START_simio_lma;
extern unsigned long __END_simio;

extern void emu_copro_process(void);

/* special definitions for vector table  */
typedef void func(void);
#ifdef __POINTER16
#define VECTOR(vec)  ( vec ),0
#else
#define VECTOR(vec)  ( vec )
#endif

/** vector table */
func *const vector[] = {			// No	Base+	Name

	/* ----- Non-Maskable Interrupts ----- */
	VECTOR( boot ),					// 0	00		Reset
	VECTOR( intAddrErr ),			// 1	04		Address misaligned
	VECTOR( intReserved ),			// 2	08		NMI
	VECTOR( emu_copro_process ),	// 3	0C		Co-processor emulation

	/* ----- Maskable Interrupts ----- */
	VECTOR( intReserved ),			// 4	10		SVD interrupt
	VECTOR( intReserved ),			// 5	14		PORT interrupt
	VECTOR( intReserved ),			// 6	18		PWG2 interrupt
	VECTOR( intReserved ),			// 7	1C		CLG interrupt
	VECTOR( intReserved ),			// 8	20		RTCA interrupt
	VECTOR( intReserved ),			// 9	24		T16 ch.0 interrupt
	VECTOR( intReserved ),			// 10	28		UART ch.0 interrupt
	VECTOR( intReserved ),			// 11	2C		T16 ch.1 interrupt
	VECTOR( intReserved ),			// 12	30		SPIA ch.0 interrupt
	VECTOR( intReserved ),			// 13	34		I2C ch.0 interrupt
	VECTOR( intReserved ),			// 14	38		T16B ch.0 interrupt
	VECTOR( intReserved ),			// 15	3C		T16B ch.1 interrupt
	VECTOR( intReserved ),			// 16	40		T16B ch.2 interrupt
	VECTOR( intReserved ),			// 17	44		UART ch.1 interrupt
	VECTOR( intReserved ),			// 18	48		T16 ch.2 interrupt
	VECTOR( intReserved ),			// 19	4C		SPIA ch.1 interrupt
	VECTOR( intReserved ),			// 20	50		SNDA interrupt
	VECTOR( intReserved ),			// 21	54		REMC2 interrupt
	VECTOR( intReserved ),			// 22	58		LCD32B interrupt
	VECTOR( intReserved ),			// 23	5C		RFC ch.3 interrupt
	VECTOR( intReserved ),			// 24	60		reserved
	VECTOR( intReserved ),			// 25	64		reserved
	VECTOR( intReserved ),			// 26	68		reserved
	VECTOR( intReserved ),			// 27	6C		reserved
	VECTOR( intReserved ),			// 28	70		reserved
	VECTOR( intReserved ),			// 29	74		reserved
	VECTOR( intReserved ),			// 30	78		reserved
	VECTOR( intReserved ),			// 31	7C		reserved
};


/****************************************************************************
 * boot function.
 *
 * @brief	First, it resets the Stack Pointer, clear the bss area,
 * 			and copy LMA to VMA.
 *			Next, it calls "main" function and calls "end" function.
 ***************************************************************************/
void boot(void)
{
	int status;

	/// Set stack pointer.
	asm("xld.a %sp, __START_stack ");


	/// Initialize section.
	clearBss();						/// - Clear the bss area.
	loadLmaToVmaData();				/// - Copy LMA data to VMA data(".data" section).
#ifdef SIM_IO_USED
	loadLmaToVmaSimIO();			/// - Copy LMA data to VMA data(".simio" section).
#endif // SIM_IO_USED

	/// Initialize library.
    _init_sys();					/// - Initialize for sys.c.
    _init_lib();					/// - Initialize for lib.c.


	asm ( "ei" );					/// Interrupt enable.

	status = main(); 				/// Call main.

//	if ( status == STATUS_NG ) {
		/***** TEST ERROR *****/
//		asm ( "brk" );
//		asm ( "nop" );
//	} else {
		/***** TEST OK *****/
//	}

	end(); 							/// Call end.
}


/* ----- static ----- */

/****************************************************************************
 * clearBss function.
 *
 * @brief	Clear ".bss" Section.
 ***************************************************************************/
static void clearBss(void)
{
	unsigned char	*addr;
	unsigned long	size;
	unsigned long	i;

	addr	= (unsigned char *)&__START_bss;
	size	= (unsigned long)&__END_bss - (unsigned long)&__START_bss;

	/// Clear ".bss" section.
	for(i = 0 ; i < size ; i++){
		*addr++ = 0x00;
	}

	return;
}

/****************************************************************************
 * loadLmaToVma function.
 *
 * @brief	This function copys LMA data to VMA data(".data" section).
 ***************************************************************************/
static void loadLmaToVmaData(void)
{
	unsigned char		*src;
	unsigned char		*dst;
	unsigned long		size;
	unsigned long		i;

	/// Load ".data" section.
	src		= (unsigned char *)&__START_data_lma;
	dst		= (unsigned char *)&__START_data;
	size	= (unsigned long)&__END_data - (unsigned long)&__START_data;

	for(i = 0 ; i < size ; i++){
		*dst++ = *src++;
	}

}

#ifdef SIM_IO_USED
/****************************************************************************
 * loadLmaToVmaSimIO function.
 *
 * @brief	This function copys LMA data to VMA data(".simio" section).
 * 			When simulated I/O is used, it calls it.
 ***************************************************************************/
static void loadLmaToVmaSimIO(void)
{
	unsigned char		*src;
	unsigned char		*dst;
	unsigned long		size;
	unsigned long		i;

	/// Load ".simio" section.
	src		= (unsigned char *)&__START_simio_lma;
	dst		= (unsigned char *)&__START_simio;
	size	= (unsigned long)&__END_simio - (unsigned long)&__START_simio;

	for(i = 0 ; i < size ; i++){
		*dst++ = *src++;
	}

}
#endif // SIM_IO_USED

/****************************************************************************
 * end function.
 *
 * @brief	This function has only infinite loop.
 ***************************************************************************/
static void end(void)
{
	/// Infinite loop.
	while ( 1 ){
		asm("halt");
	}
}

/* ----- Non Maskable Interrupt Handler ----- */

/****************************************************************************
 * intAddrErr function.
 *
 * @brief	Interrupt handler for "Address misaligned".
 ***************************************************************************/
void intAddrErr(void)
{
	/// Undecid processing.
	asm("halt");
}

/****************************************************************************
 * intReserved function.
 *
 * @brief	Interrupt handler for "Reserved".
 ***************************************************************************/
void intReserved(void)
{
	/// Infinite loop.
	while ( 1 ){
		;
	}
}
